package.path = './?.lua'
package.cpath = '../bin/lua-?.dll'

local sdl = require('sdl')
local gl = require('opengl')

local windowWidth = 400
local windowHeight = 300
local xrot = 0
local yrot = 0
local zrot = 0

gl.Create()
sdl.Create(windowWidth, windowHeight, false)

function loadTexture(filename)
  local data, width, height, bpp = gl.LoadImageData(filename)
  if data then
    texture = gl.GenTextures(1)
    gl.BindTexture(gl.TEXTURE_2D, texture[1]);
    local internalFormat
    local imageFormat    
    if 3 < bpp then
      internalFormat = gl.RGBA8
      imageFormat = gl.RGBA
    else
      internalFormat = gl.RGB8
      imageFormat = gl.RGB  
    end
    
    gl.TexImage2D(gl.TEXTURE_2D, 0, internalFormat, width, height, 0, imageFormat, data);
    gl.TexParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.LINEAR);
    gl.TexParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.LINEAR);
  end
end

function init()
  loadTexture('../images/lua.png')
  gl.Enable(gl.TEXTURE_2D)						
	gl.ShadeModel(gl.SMOOTH)						
	gl.ClearColor(0, 0, 0, 0.5)				
	gl.ClearDepth(1)								
	gl.Enable(gl.DEPTH_TEST)						
	gl.DepthFunc(gl.LEQUAL)						
	gl.Hint(gl.PERSPECTIVE_CORRECTION_HINT, gl.NICEST)
end

function resize(width, height)
  gl.Viewport(0, 0, width, height)
  gl.MatrixMode(gl.PROJECTION)
  gl.LoadIdentity()
	gl.uPerspective(45, width/height, 0.1, 100)
	gl.MatrixMode(gl.MODELVIEW)
	gl.LoadIdentity()
end

fpsStart = os.clock()
fpsCounter = 0

function idle()
  xrot = xrot + 0.3
  yrot = yrot + 0.2
  zrot = zrot + 0.4
  local currTime = os.clock()
  if currTime - fpsStart > 1 then
    print('FPS:', fpsCounter)
    fpsCounter = 0
    fpsStart = currTime
  end
end

function draw()
  gl.Clear()
  
	gl.LoadIdentity()
	gl.Translate(0, 0, -5)

	gl.Rotate(xrot, 1, 0, 0)
	gl.Rotate(yrot, 0, 1, 0)
	gl.Rotate(zrot, 0, 0, 1)

	gl.BindTexture(gl.TEXTURE_2D, texture[1])
  
	gl.Begin(gl.QUADS)
		-- Front Face
    gl.Color(1, 0, 0)
		gl.TexCoord2(0, 0); gl.Vertex(-1,  1,  1);
		gl.TexCoord2(1, 0); gl.Vertex( 1,  1,  1);
		gl.TexCoord2(1, 1); gl.Vertex( 1, -1,  1);
		gl.TexCoord2(0, 1); gl.Vertex(-1, -1,  1);
		-- Back Face
    gl.Color(0, 1, 0)
		gl.TexCoord2(0, 0); gl.Vertex( 1,  1, -1);
		gl.TexCoord2(1, 0); gl.Vertex(-1,  1, -1);
		gl.TexCoord2(1, 1); gl.Vertex(-1, -1, -1);
		gl.TexCoord2(0, 1); gl.Vertex( 1, -1, -1);
		-- Top Face  
    gl.Color(0, 0, 1)
		gl.TexCoord2(0, 0); gl.Vertex(-1,  1, -1);
		gl.TexCoord2(1, 0); gl.Vertex( 1,  1, -1);
		gl.TexCoord2(1, 1); gl.Vertex( 1,  1,  1);
		gl.TexCoord2(0, 1); gl.Vertex(-1,  1,  1);
		-- Bottom Face
    gl.Color(1, 1, 0)
		gl.TexCoord2(0, 0); gl.Vertex(-1, -1,  1);
		gl.TexCoord2(1, 0); gl.Vertex( 1, -1,  1);
		gl.TexCoord2(1, 1); gl.Vertex( 1, -1, -1);
		gl.TexCoord2(0, 1); gl.Vertex(-1, -1, -1);
		-- Right face
    gl.Color(1, 0, 1)    
		gl.TexCoord2(0, 0); gl.Vertex( 1,  1,  1);
		gl.TexCoord2(1, 0); gl.Vertex( 1,  1, -1);
		gl.TexCoord2(1, 1); gl.Vertex( 1, -1, -1);
		gl.TexCoord2(0, 1); gl.Vertex( 1, -1,  1);
		-- Left Face 
    gl.Color(0, 1, 1)      
		gl.TexCoord2(0, 0); gl.Vertex(-1,  1, -1);
		gl.TexCoord2(1, 0); gl.Vertex(-1,  1,  1);
		gl.TexCoord2(1, 1); gl.Vertex(-1, -1,  1);
		gl.TexCoord2(0, 1); gl.Vertex(-1, -1, -1);
  gl.End()     

  fpsCounter = fpsCounter + 1
end

init()
resize(windowWidth, windowHeight)
sdl.Run()
