#include "SDL/SDL.h"
#include "LuaUtil.h"
#include <iostream>

static SDL_Surface* screen = 0;
static int screenWidth, screenHeight;
static bool fullScreen = false;
static int delay = 1;

static int create(lua_State* L)
{
  // stack: width height fullscreen
  if(!fromLua(L, 1, screenWidth) ||
     !fromLua(L, 2, screenHeight))
  {
    luaL_error(L, "Invalid arguments!\n");
  }

  SDL_Init(SDL_INIT_VIDEO);
  SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  //   Unicode
  SDL_EnableUNICODE(1);

  fromLua(L, 3, fullScreen);
  if(fullScreen)
  {
    screen = SDL_SetVideoMode(screenWidth, screenHeight, 32, SDL_OPENGL | SDL_FULLSCREEN);
  }
  else
  {
    screen = SDL_SetVideoMode(screenWidth, screenHeight, 32, SDL_OPENGL | SDL_RESIZABLE);
  }
  SDL_WM_SetCaption("SDL OPENGL TEST", "SDL OPENGL TEST");

  return 0;
}

static void idle(lua_State* L)
{
  // stack:
  LuaStackChecker sc(L, __FILE__, __LINE__);
  lua_getglobal(L, "idle");   // stack: idle
  if(lua_isfunction(L, -1))
  {
    lua_call(L, 0, 0); // stack:
  }
  else
  {
    lua_pop(L, 1); // stack:
  }
}

static void draw(lua_State* L)
{
  // stack:
  LuaStackChecker sc(L, __FILE__, __LINE__);
  lua_getglobal(L, "draw");   // stack: draw
  if(lua_isfunction(L, -1))
  {
    lua_call(L, 0, 0); // stack:
  }
  else
  {
    lua_pop(L, 1); // stack:
  }
}

static int run(lua_State* L)
{
  SDL_Event event;
  bool done = false;
  while(!done)
  {
    idle(L);
    while(SDL_PollEvent(&event))
    {
      switch(event.type)
	      {
	      case SDL_QUIT:
		      done = true;
		      break;
	      case SDL_VIDEORESIZE:
          //resize(event.resize.w, event.resize.h);
		      break;
	      case SDL_MOUSEMOTION:
          //mouseMove(event.motion.x, event.motion.y);
		      break;
	      case SDL_MOUSEBUTTONUP:
//          mouseUp(event.button.button);
          break;
	      case SDL_MOUSEBUTTONDOWN:
//          mouseDown(event.button.button);
		      break;
	      case SDL_KEYUP:
//          keyUp(SDL_GetKeyName(event.key.keysym.sym), event.key.keysym.unicode);
          break;
	      case SDL_KEYDOWN:
//          keyDown(SDL_GetKeyName(event.key.keysym.sym), event.key.keysym.unicode);
		      break;
	      default:
          break;
        }
    }
    draw(L);
    SDL_GL_SwapBuffers();
    SDL_Delay(delay);
  }
  SDL_Quit();

  return 0;
}

static const luaL_Reg sdl[] = {
  {"Create", create},
  {"Run", run},
  {0, 0},
};

extern "C" __declspec(dllexport) int luaopen_sdl(lua_State* L)
{
	luaL_register(L, "sdl", sdl);
  return 1;
}
